<?php

// Copyright 2007-2009 Jonathan Brown


define("OP_VIDEO_DEFAULT_ALLOW_REPLACE", 0);
define("OP_VIDEO_DEFAULT_ALLOW_EMBED", 0);


/*
      FIELD
*/


function op_video_field_info() {

  return array(
    'op_video' => array('label' => 'OpenPackage Video')
  );
}


function _op_video_get_params_id($field) {
  $params_ids = variable_get('op_video_field_params_ids', array());
  return $params_ids[$field['field_name']];
}


function _op_video_field_load_params($field) {
  $params_ids = variable_get('op_video_field_params_ids', array());

  if(!$params_ids[$field['field_name']]) {
    $params_ids[$field['field_name']] = _op_video_transcoding_params_insert();
    variable_set('op_video_field_params_ids', $params_ids);
  }
  
  return _op_video_transcoding_params_load($params_ids[$field['field_name']]);
}


function _op_video_field_retranscode_all($field) {

  $table = 'content_type_' . $field['type_name'];
  $video_id_field = $field['field_name'] . '_video_id';

  $result = db_query("
    SELECT {op_videos}.video_id
    FROM {{$table}}
    INNER JOIN {op_videos}
      ON {{$table}}.$video_id_field = {op_videos}.video_id
    WHERE {op_videos}.source_file_id IS NOT NULL
  ");

  while($video = db_fetch_array($result)) {
    _op_video_retranscode_video($video['video_id']);
  }

  $msg = t('Retranscoding all field %label videos.', array('%label' => $field['label']));
  drupal_set_message($msg);
  watchdog('op_video', $msg);
}


function _op_video_field_store_params($field) {
  $params_id = _op_video_get_params_id($field);
  _op_video_transcoding_params_update($params_id, $field);
  
  if($field['retranscode_all']) {
    _op_video_field_retranscode_all($field);
  }
}


function _op_video_get_selector_ids($field_name) {

  $selector_ids = variable_get('op_video_field_selector_ids', array());

  foreach(array('source', 'transcoded', 'cache') as $selector) {

    if($selector_ids[$field_name][$selector]) {
    
      // this will fix a situation where storage_api was uninstalled, but op_video was not
    
      $new = !db_result(db_query("
        SELECT COUNT(*)
        FROM {storage_selectors}
        WHERE selector_id = %d
      ",
        $selector_ids[$field_name][$selector]
      ));
    }
    else {    
      $new = TRUE;
    }
    
    if($new) {
      $selector_ids[$field_name][$selector] = storage_api_new_selector_id();
      $update = TRUE;
    }
  }
  
  if($update)
    variable_set('op_video_field_selector_ids', $selector_ids);
  
  return $selector_ids[$field_name];
}


function _op_video_selectors_fieldset($field_name) {

  $selector_ids = _op_video_get_selector_ids($field_name);
  
  $fieldset = array(
    '#type' => 'fieldset',
    '#tree' => TRUE,
    '#title' => t('Storage classes'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );
  
  $fieldset['source'] = storage_api_selector_item($selector_ids['source'], t('Source class'),
    t('Source video and custom preview images.'));
    
  $fieldset['transcoded'] = storage_api_selector_item($selector_ids['transcoded'], t('Transcoded class'),
    t('Transcoded video and captured preview images.'));
    
  $fieldset['cache'] = storage_api_selector_item($selector_ids['cache'], t('Cache class'),
    t('Cached scaled images.'));
      
  return $fieldset;
}


function op_video_field_settings($op, $field) {

  switch ($op) {

    case 'form':
    
      $description = t("Allow the video the be embedded on other websites.") . '<br />';
      $description .= t("Users with <em>administer video</em> permission can always do this.");

      $form['allow_embed'] = array(
        '#type' => 'checkbox',
        '#title' => 'Allow embedding',
        '#default_value' => isset($field['allow_embed']) ? $field['allow_embed'] : OP_VIDEO_DEFAULT_ALLOW_EMBED,
        '#description' => $description
      );
      
      $form['preview_image_settings'] = _op_video_preview_image_settings_fieldset($field);
      $form['player_settings'] = _op_video_player_settings_fieldset($field);
      
      $params = _op_video_field_load_params($field);
      $form['transcoding_params'] = _op_video_transcoding_params_fieldset($params);
      
      $form['selectors'] = _op_video_selectors_fieldset($field['field_name']);

      return $form;


    case 'validate':
    
      _op_video_preview_image_settings_validate($field);
      _op_video_player_settings_validate($field);
      _op_video_transcoding_params_validate($field);
      
      if(!form_get_errors()) {    // $form data is not passed to 'save' op below
        _op_video_field_store_params($field);
        storage_api_selector_submit($field['selectors']['source']);
        storage_api_selector_submit($field['selectors']['transcoded']);
        storage_api_selector_submit($field['selectors']['cache']);
      }
      return;


    case 'save':
    
      _op_video_purge_image_cache();
      $preview_image_fields = _op_video_player_settings_fields();
      $player_fields = _op_video_preview_image_settings_fields();
      return array_merge(array('source_storage_api_selector_id', 'transcoded_storage_api_selector_id', 'preview_images_storage_api_selector_id', 'allow_embed'), $preview_image_fields, $player_fields);


    case 'database columns':

      return array(
        'video_id' => array('type' => 'int', 'unsigned' => TRUE, 'not null' => TRUE)
      );

      
    case 'tables':
    
      $tables = content_views_field_tables($field);
      
      $table_name = array_shift(array_keys($tables));
      $field_name = array_shift(array_keys($tables[$table_name]['fields']));
      
      unset($tables[$table_name]['fields']);
      
      $tables[$field['field_name'] . '_op_videos'] = array(
        'name' => 'op_videos',
        'join' => array(
          'left' => array(
            'table' => $table_name,
            'field' => $field_name
          ),
          'right' => array(
            'field' => 'video_id'
          )
        ),
        'fields' => _op_video_views_op_videos_fields($field['widget']['label']),
        'sorts' => _op_video_views_op_videos_sorts($field['widget']['label']),
        'filters' => _op_video_views_op_videos_filters($field['widget']['label'])
      );
      
      $tables += storage_api_views_tables2($field['field_name'] . '_op_videos', 'source_file_id', 
        $field['field_name'] . '_source', $field['widget']['label'] . ' (source)');

      $tables += storage_api_views_tables2($field['field_name'] . '_op_videos', 'flv_file_id', 
        $field['field_name'] . '_transcoded', $field['widget']['label'] . ' (transcoded)');

      return $tables;
  }
}


function op_video_field($op, &$node, $field, &$node_field, $teaser, $page) {

  if($node_field[0]['custom_image_file_id'])
    $node_field[0]['preview_image']['preview_image'] = 'custom';

  switch ($op) {

    case 'insert':
      $params_id = _op_video_get_params_id($field);

      $node_field[0]['video_id'] = 
        _op_video_create_video($node_field[0]['source_file_id'], $params_id, 
          $node_field[0]['preview_image']['preview_image'], $node_field[0]['custom_image_file_id'],
          $field['field_name'], $node->nid, $node_field[0]['flv_file_id'], $node_field[0]['duration']);
      
      break;
  
      
    case 'update':
      $video = _op_video_load($node_field[0]['video_id']);
      
      if($node_field[0]['upload']['delete']) {
        _op_video_delete($video);
        $node_field[0]['video_id'] = 0;
      }
      else {
      
        if($node_field[0]['source_file_id']) {
          _op_video_update_source_file($video, $node_field[0]['source_file_id']);
          drupal_set_message('The new video will be transcoded.');
        }
        else if($node_field[0]['retranscode']) {
          _op_video_retranscode_video($node_field[0]['video_id']);
          drupal_set_message('The video will be retranscoded.');
        }
        
        if($node_field[0]['preview_image']['upload']['delete']) {
          _op_video_update_custom_image($video, 0);
          
          if($node_field[0]['preview_image']['preview_image'] == 'custom')
            $node_field[0]['preview_image']['preview_image'] = $field['preview_image_image'];
        }
        else {
        
          if($node_field[0]['custom_image_file_id'])
            _op_video_update_custom_image($video, $node_field[0]['custom_image_file_id']);
        }

        if($node_field[0]['preview_image']['preview_image'] && 
          ($node_field[0]['preview_image']['preview_image'] != $video['preview_image'])) {
          _op_video_set_preview_image($node_field[0]['video_id'], $node_field[0]['preview_image']['preview_image']);
        }
      }

      break;


    case 'delete':
      _op_video_delete($node_field[0]['video_id']);
      break;
  }
}


/*
      WIDGET
*/


function op_video_widget_info() {

  return array(
    'upload' => array(
      'label' => t('Upload / Acquire from URL / Filepath'),
      'field types' => array('op_video'),
    )
  );
}


function op_video_widget_settings($op, $widget) {

  switch ($op) {

    case 'form':
    
      $description = t("Allow the video the be replaced / deleted after the node has been created.") . "<br />\n";
      $description .= t("Users with <em>administer video</em> permission can always do this.");

      $form['allow_replace'] = array(
        '#type' => 'checkbox',
        '#title' => 'Allow replacement',
        '#default_value' => isset($widget['allow_replace']) ? $widget['allow_replace'] : OP_VIDEO_DEFAULT_ALLOW_REPLACE,
        '#description' => $description
      );
      
      return $form;

    case 'save':
      return array('allow_replace');

    case 'callbacks':
      return array(
        'view' => CONTENT_CALLBACK_CUSTOM,
      );
  }
}


function _op_video_widget_upload_form_item($text, $field, $existing, $filepath, $required = FALSE) {

  if(!count($_POST)) {

    // this is a fresh form - delete old preview video if it exists

    $file = file_save_upload($field['field_name'], file_directory_temp());

    if($file)
      @unlink($file->filepath);
  }

  if($existing) {
  
    if(!$required) {
      $item['delete'] = array(
        '#type' => 'checkbox',
        '#title' => t('Delete this') . ' ' . t($text),
      );
    }
     
    $item[] = array(
      '#value' => '<div style="clear: both;"></div>'
    );
  }
  
  $file = file_check_upload($field['field_name']);

  if($file) {
    $info  = 'Filename: ' . $file->filename . '<br />';
    $info .= 'Filesize: ' . storage_format_size($file->filesize);
    $item['uploaded'] = array(
      '#type' => 'fieldset',
      '#title' => t('Uploaded') . ' ' . t($text)
    );
    $item['uploaded'][] = array(
      '#value' => $info
    );
    $item['uploaded']['ignore'] = array(
      '#type' => 'checkbox',
      '#title' => t('Ignore')
    );
  }
  
  $item[$field['field_name']] = array(
    '#type' => 'file', 
    '#name' => 'files[' . $field['field_name'] . ']',
    '#description' => 'Max size: ' . storage_format_size(file_upload_max_size())
  );

  if(!$existing && !$file)
    $item[$field['field_name']]['#title'] = t('Upload') . ' ' . t($text);
  else
    $item[$field['field_name']]['#title'] = t('Upload new') . ' ' . t($text);
  
  $item['acquire_url'] = array(
    '#type' => 'textfield',
    '#title' => 'Acquire from URL'
  );

  if(user_access('administer video')) {
    $item['filepath'] = array(
      '#type' => 'textfield',
      '#title' => 'Filepath',
      '#autocomplete_path' => 'op_video/path_autocomplete',
      '#default_value' => $filepath
    );
  }
  
  return $item;
}


function _op_video_widget_preview_image($field, $node_field) {

  $item = array(
    '#type' => 'fieldset',
    '#title' => t('Preview image')
  );
  
  if($node_field[0]['video_id']) {
    $video = _op_video_load($node_field[0]['video_id']);
  }
  
  if($video['start_image_file_id'])
    $options['start'] = _op_video_render_preview_image($video, 80, 60, NULL, 'start') . ' - start of video';
  else
    $options['start'] = 'Start of video';
  
  if($video['third_image_file_id'])
    $options['third'] = _op_video_render_preview_image($video, 80, 60, NULL, 'third') . ' - @ 1/3 of video duration';
  else
    $options['third'] = '@ 1/3 of video duration';

  $existing = $video['custom_image_file_id'] != 0;

  if($existing)
    $options['custom'] = _op_video_render_preview_image($video, 80, 60, NULL, 'custom') . ' - custom image';

  $item['preview_image'] = array(
    '#type' => 'radios',
    '#default_value' => isset($node_field[0]['preview_image']) ? $node_field[0]['preview_image'] : 
      (isset($field['preview_image_image']) ? $field['preview_image_image'] : OP_VIDEO_DEFAULT_PREVIEW_IMAGE_IMAGE),
    '#options' => $options
  );

  $field['field_name'] .= '_image';
  $item['upload'] = _op_video_widget_upload_form_item('custom image', $field, $existing, 
    $video['custom_image_file']['filepath']);

  return $item;
}


function _op_video_widget_upload_validate($name, $preview) {

  if($preview) {
    $file = file_save_upload($name);
  }
  else {
    $file = file_check_upload($name);
  }

  if($file) {
    if($_SESSION['file_uploads'][$name]->filepath != $file->filepath)
      @unlink($_SESSION['file_uploads'][$name]->filepath);
  }
  
  return $file;
}


function op_video_widget($op, &$node, $field, &$node_field) {

  switch ($op) {

    case 'prepare form values':
      $video = _op_video_load($node_field[0]['video_id']);
      $node_field[0]['preview_image'] = $video['preview_image'];
      break;


    case 'form':

      $form[$field['field_name']]['#weight'] = $field['widget']['weight'];
      $form[$field['field_name']]['#tree'] = TRUE;
      $form[$field['field_name']][0] = array(
        '#type' => 'fieldset',
        '#title' => t($field['widget']['label']),
        '#description' => content_filter_xss($field['widget']['description'])
      );
      
      $existing = $node_field[0]['video_id'];
      
      if($existing) {
      
        $form[$field['field_name']][0]['video_id'] = array(
          '#type' => 'value',
          '#value' => $node_field[0]['video_id']
        );

        $video = _op_video_load($node_field[0]['video_id']);
        $filepath = $video['source_file']['filepath'];
        
        $info = _op_video_render_preview_image($video, 240, 180, array('style="float: left; margin-right: 10px;"'));
      
        $info .= 'Filename: ' . $video['source_file']['filename'] . '<br />';

        if($video['source_file']['size']) {
          $info .= 'Size: ' . storage_format_size($video['source_file']['size']) . '<br />';
        }
        
        if($video['duration'])
          $info .= 'Duration: ' . format_interval($video['duration']) . '<br />';
        
        $form[$field['field_name']][0][] = array(
          '#value' => '<p>' . $info . '</p>'
        );

        if(user_access('administer video') && $video['source_file_id']) {
          
          $form[$field['field_name']][0]['retranscode'] = array(
            '#type' => 'checkbox', 
            '#title' => t('Retranscode this video')
          );
        }
      }
      
      if(!$existing || $field['widget']['allow_replace'] || user_access('administer video')) {
        $form[$field['field_name']][0]['upload'] = 
          _op_video_widget_upload_form_item('video', $field, $existing, $filepath, $field['required']);
      }
      else {
        $form[$field['field_name']][0][] = array(
          '#value' => '<div style="clear: both;"></div>'
        );
      }      

      if(isset($field['preview_image_user_selectable']) ? $field['preview_image_user_selectable'] :
        OP_VIDEO_DEFAULT_PREVIEW_IMAGE_USER_SELECTABLE) // || user_access('administer video'))
      {
        $form[$field['field_name']][0]['preview_image'] = _op_video_widget_preview_image($field, $node_field);
      }

      $form['#attributes']['enctype'] = 'multipart/form-data';
      
      return $form;


    case 'validate':
    
      $file = _op_video_widget_upload_validate($field['field_name'], $node->op == t('Preview'));

      if(!$file && !$node_field[0]['upload']['acquire_url'] && $node_field[0]['upload']['filepath']) {
        if(!is_file($node_field[0]['upload']['filepath'])) {
          form_set_error($field['field_name'] . '][0][upload][filepath', t('File not found.'));
        }
      }

      if(!$node_field[0]['video_id'] && $field['required'] &&
        !($file && !$node_field[0]['upload']['uploaded']['ignore']) &&
        !$node_field[0]['upload']['acquire_url'] &&
        !$node_field[0]['upload']['filepath'])
      {
        form_set_error($field['field_name'] . '][0][upload]['. $field['field_name'], t('You need to upload a video.'));
      }

      $file = _op_video_widget_upload_validate($field['field_name'] . '_image', $node->op == t('Preview'));
      
      if(!$file && !$node_field[0]['preview_image']['upload']['acquire_url'] && 
        $node_field[0]['preview_image']['upload']['filepath'])
      {
        if(!is_file($node_field[0]['preview_image']['upload']['filepath'])) {
          form_set_error($field['field_name'] . '][0][preview_image][upload][filepath', t('File not found.'));
        }
      }
      
      break;


    case 'submit':
    
      $selector_ids = _op_video_get_selector_ids($field['field_name']);

      if($node_field[0]['video_id'])
        $video = _op_video_load($node_field[0]['video_id']);

      if($node_field[0]['upload']['delete'] || 
        $node_field[0]['upload']['uploaded']['ignore'])
      {
        $file = file_save_upload($field['field_name'], file_directory_temp());

        if($file)
          @unlink($file->filepath);
          
        unset($file);
      }
      else {
        $file = file_check_upload($field['field_name']);
      }
      
      $options = array(
        'module' => 'op_video',
        'type' => 'source video',
        'force_download' => TRUE
      );

      if($file) {
      
        $node_field[0]['source_file_id'] = 
          storage_api_add_file_from_upload($field['field_name'], $selector_ids['source'], $options);
      }
      else if($node_field[0]['upload']['acquire_url']) {

        $node_field[0]['source_file_id'] = 
          storage_api_add_file_from_url($node_field[0]['upload']['acquire_url'], $selector_ids['source'], $options);
      }
      else if($node_field[0]['upload']['filepath']) {

        $node_field[0]['source_file_id'] = 
          storage_api_add_file_from_filepath($node_field[0]['upload']['filepath'], $selector_ids['source'], $options);
      }
      
      if($video || $node_field[0]['source_file_id'] || 
        $node_field[0]['upload']['acquire_url'] || $node_field[0]['upload']['filepath']) {
      
        if($node_field[0]['preview_image']['upload']['delete'] || 
          $node_field[0]['preview_image']['upload']['uploaded']['ignore'])
        {
          $file = file_save_upload($field['field_name'] . '_image', file_directory_temp());

          if($file)
            @unlink($file->filepath);
          
          unset($file);
        }
        else {
          $file = file_check_upload($field['field_name'] . '_image');
        }
        
        $options = array(
          'module' => 'op_video',
          'type' => 'custom preview image'
        );

        if($file) {
          $node_field[0]['custom_image_file_id'] = 
            storage_api_add_file_from_upload($field['field_name'] . '_image', $selector_ids['source'], $options);
        }
        else if($node_field[0]['preview_image']['upload']['acquire_url']) {

          $node_field[0]['custom_image_file_id'] = 
            storage_api_add_file_from_url($node_field[0]['upload']['acquire_url'], $selector_ids['source'], $options);
        }
        else if($node_field[0]['preview_image']['upload']['filepath']) {

          $node_field[0]['custom_image_file_id'] = 
            storage_api_add_file_from_filepath($node_field[0]['upload']['filepath'], $selector_ids['source'], $options);
        }
      }

      break;
  }
}


/*
      FORMATTERS
*/


function op_video_field_formatter_info() {

  return array(
    'default' => array(
      'label' => 'Inline player',
      'field types' => array('op_video'),
    ),
    'preview-image-link' => array(
      'label' => 'Preview image - link to node',
      'field types' => array('op_video'),
    ),
    'preview-image-layer' => array(
      'label' => 'Preview image - play in layer',
      'field types' => array('op_video'),
    )
  );
}


function op_video_field_formatter($field, $item, $formatter, $node) {

  _op_video_set_render_settings($field);
  
  $video = _op_video_load($item['video_id']);
  
  return _op_video_render_video($video, $formatter, $node->nid);
}


function _op_video_get_first_video_field($node) {
  $content_type_info = _content_type_info();
  
  foreach((array)$node as $field => $value) {
    if($content_type_info['fields'][$field]['type'] == 'op_video') {
      return $field;
    }
  }
}


function _op_video_get_video_fields($node) {
  $content_type_info = _content_type_info();
  $fields = array();
  
  foreach((array)$node as $field => $value) {
    if($content_type_info['fields'][$field]['type'] == 'op_video') {
      $fields[] = $field;
    }
  }
  
  return $fields;
}


function _op_video_get_first_video($node) {

  $field = _op_video_get_first_video_field($node);
  
  if($field)
     return _op_video_load($node->{$field}[0]['video_id']);
}


function op_video_nodeapi($node, $op, $teaser, $page) {

  if($op != 'rss item')
    return;
    
  $video = _op_video_get_first_video($node);

  if($video['source_file']) {

    $rss_items[] = array(
      'key' => 'enclosure',
      'attributes' => array(
        'url' => storage_api_serve_url($video['source_file'], TRUE),
        'length' => $video['source_file']['size'],
        'type' => $video['source_file']['mimetype']
      )
    );
    
    return $rss_items;
  }
}

